@@ -21,7 +21,14 @@ use goblin::{
     elf::{header::*, reloc::*, section_header::*, Elf},
     error::Error as GoblinError,
 };
-use std::{collections::BTreeMap, fmt::Debug, mem, ops::Range, pin::Pin, str};
+use std::{
+    collections::{btree_map::Entry, BTreeMap},
+    fmt::Debug,
+    mem,
+    ops::Range,
+    pin::Pin,
+    str,
+};
 
 /// Error definitions
 #[derive(Debug, thiserror::Error, PartialEq, Eq)]
@@ -121,23 +128,31 @@ pub fn hash_bpf_function(pc: usize, name: &str) -> u32 {
 }
 
 /// Register a symbol or throw ElfError::SymbolHashCollision
-pub fn register_bpf_function(
+pub fn register_bpf_function<T: AsRef<str> + ToString>(
     bpf_functions: &mut BTreeMap<u32, (usize, String)>,
     pc: usize,
-    name: &str,
+    name: T,
     enable_symbol_and_section_labels: bool,
 ) -> Result<u32, ElfError> {
-    let hash = hash_bpf_function(pc, name);
-    let name = if enable_symbol_and_section_labels {
-        name.to_string()
-    } else {
-        String::default()
-    };
-    if let Some(entry) = bpf_functions.insert(hash, (pc, name)) {
-        if entry.0 != pc {
-            return Err(ElfError::SymbolHashCollision(hash));
+    let hash = hash_bpf_function(pc, name.as_ref());
+    match bpf_functions.entry(hash) {
+        Entry::Vacant(entry) => {
+            entry.insert((
+                pc,
+                if enable_symbol_and_section_labels {
+                    name.to_string()
+                } else {
+                    String::default()
+                },
+            ));
+        }
+        Entry::Occupied(entry) => {
+            if entry.get().0 != pc {
+                return Err(ElfError::SymbolHashCollision(hash));
+            }
         }
     }
+
     Ok(hash)
 }
 
@@ -215,6 +230,11 @@ struct SectionInfo {
     vaddr: u64,
     offset_range: Range<usize>,
 }
+impl SectionInfo {
+    fn mem_size(&self) -> usize {
+        mem::size_of::<Self>().saturating_add(self.name.capacity())
+    }
+}
 
 /// Elf loader/relocator
 #[derive(Debug, PartialEq)]
@@ -398,8 +418,7 @@ impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> {
             vaddr: text_section.sh_addr.saturating_add(ebpf::MM_PROGRAM_START),
             offset_range: text_section.file_range().unwrap_or_default(),
         };
-        if (config.reject_section_virtual_address_file_offset_mismatch
-            && text_section.sh_addr != text_section.sh_offset)
+        if (config.reject_broken_elfs && text_section.sh_addr != text_section.sh_offset)
             || text_section_info.vaddr > ebpf::MM_STACK_START
         {
             return Err(ElfError::ValueOutOfBounds);
@@ -451,8 +470,7 @@ impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> {
                 let vaddr = section_header
                     .sh_addr
                     .saturating_add(ebpf::MM_PROGRAM_START);
-                if (config.reject_section_virtual_address_file_offset_mismatch
-                    && section_header.sh_addr != section_header.sh_offset)
+                if (config.reject_broken_elfs && section_header.sh_addr != section_header.sh_offset)
                     || vaddr > ebpf::MM_STACK_START
                 {
                     return Err(ElfError::ValueOutOfBounds);
@@ -468,8 +486,7 @@ impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> {
             })
             .collect::<Result<Vec<_>, ElfError>>()?;
         if ro_alloc_length > elf_bytes.len()
-            || (config.reject_section_virtual_address_file_offset_mismatch
-                && ro_fill_length > ro_alloc_length)
+            || (config.reject_broken_elfs && ro_fill_length > ro_alloc_length)
         {
             return Err(ElfError::ValueOutOfBounds);
         }
@@ -498,6 +515,40 @@ impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> {
         })
     }
 
+    /// Calculate the total memory size of the executable
+    #[rustfmt::skip]
+    pub fn mem_size(&self) -> usize {
+        let total = mem::size_of::<Self>()
+            // elf bytres
+            .saturating_add(self.elf_bytes.mem_size())
+            // ro section
+            .saturating_add(self.ro_section.capacity())
+            // text section info
+            .saturating_add(self.text_section_info.mem_size())
+            // bpf functions
+            .saturating_add(mem::size_of_val(&self.bpf_functions))
+            .saturating_add(self.bpf_functions
+            .iter()
+            .fold(0, |state: usize, (_, (val, name))| state
+                .saturating_add(mem::size_of_val(&val)
+                .saturating_add(mem::size_of_val(&name)
+                .saturating_add(name.capacity())))))
+            // syscall symbols
+            .saturating_add(mem::size_of_val(&self.syscall_symbols))
+            .saturating_add(self.syscall_symbols
+            .iter()
+            .fold(0, |state: usize, (val, name)| state
+                .saturating_add(mem::size_of_val(&val)
+                .saturating_add(mem::size_of_val(&name)
+                .saturating_add(name.capacity())))))
+            // syscall registry
+            .saturating_add(self.syscall_registry.mem_size())
+            // compiled programs
+            .saturating_add(self.compiled_program.as_ref().map_or(0, |program| program.mem_size()));
+
+        total as usize
+    }
+
     // Functions exposed for tests
 
     /// Fix-ups relative calls
@@ -521,11 +572,16 @@ impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> {
                         i.saturating_add(ebpf::ELF_INSN_DUMP_OFFSET),
                     ));
                 }
-                let name = format!("function_{}", target_pc);
+                let name = if enable_symbol_and_section_labels {
+                    format!("function_{}", target_pc)
+                } else {
+                    String::default()
+                };
+
                 let hash = register_bpf_function(
                     bpf_functions,
                     target_pc as usize,
-                    &name,
+                    name,
                     enable_symbol_and_section_labels,
                 )?;
                 insn.imm = hash as i64;
@@ -533,14 +589,14 @@ impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> {
                 let checked_slice = elf_bytes
                     .get_mut(offset..offset.saturating_add(ebpf::INSN_SIZE))
                     .ok_or(ElfError::ValueOutOfBounds)?;
-                checked_slice.copy_from_slice(&insn.to_vec());
+                checked_slice.copy_from_slice(&insn.to_array());
             }
         }
         Ok(())
     }
 
     /// Validates the ELF
-    pub fn validate(config: &Config, elf: &Elf, elf_bytes: &[u8]) -> Result<(), ElfError> {
+    pub fn validate(_config: &Config, elf: &Elf, elf_bytes: &[u8]) -> Result<(), ElfError> {
         if elf.header.e_ident[EI_CLASS] != ELFCLASS64 {
             return Err(ElfError::WrongClass);
         }
@@ -574,10 +630,9 @@ impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> {
 
         for section_header in elf.section_headers.iter() {
             if let Some(name) = elf.shdr_strtab.get_at(section_header.sh_name) {
-                if config.reject_all_writable_sections
-                    && (name.starts_with(".bss")
-                        || (section_header.is_writable()
-                            && (name.starts_with(".data") && !name.starts_with(".data.rel"))))
+                if name.starts_with(".bss")
+                    || (section_header.is_writable()
+                        && (name.starts_with(".data") && !name.starts_with(".data.rel")))
                 {
                     return Err(ElfError::WritableSectionNotSupported(name.to_owned()));
                 } else if name == ".bss" {
@@ -766,7 +821,7 @@ impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> {
                             .entry(symbol.st_name)
                             .or_insert_with(|| (ebpf::hash_symbol_name(name.as_bytes()), name))
                             .0;
-                        if config.reject_unresolved_syscalls
+                        if config.reject_broken_elfs
                             && syscall_registry.lookup_syscall(hash).is_none()
                         {
                             return Err(ElfError::UnresolvedSymbol(
@@ -980,7 +1035,7 @@ mod test {
     #[test]
     fn test_fixup_relative_calls_back() {
         // call -2
-        let mut bpf_functions: BTreeMap<u32, (usize, String)> = BTreeMap::new();
+        let mut bpf_functions = BTreeMap::new();
         #[rustfmt::skip]
         let mut prog = vec![
             0xb7, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -1004,7 +1059,7 @@ mod test {
         assert_eq!(*bpf_functions.get(&hash).unwrap(), (4, name));
 
         // call +6
-        let mut bpf_functions: BTreeMap<u32, (usize, String)> = BTreeMap::new();
+        let mut bpf_functions = BTreeMap::new();
         prog.splice(44.., vec![0xfa, 0xff, 0xff, 0xff]);
         ElfExecutable::fixup_relative_calls(true, &mut bpf_functions, &mut prog).unwrap();
         let name = "function_0".to_string();
@@ -1023,7 +1078,7 @@ mod test {
     #[test]
     fn test_fixup_relative_calls_forward() {
         // call +0
-        let mut bpf_functions: BTreeMap<u32, (usize, String)> = BTreeMap::new();
+        let mut bpf_functions = BTreeMap::new();
         #[rustfmt::skip]
         let mut prog = vec![
             0x85, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -1047,7 +1102,7 @@ mod test {
         assert_eq!(*bpf_functions.get(&hash).unwrap(), (1, name));
 
         // call +4
-        let mut bpf_functions: BTreeMap<u32, (usize, String)> = BTreeMap::new();
+        let mut bpf_functions = BTreeMap::new();
         prog.splice(4..8, vec![0x04, 0x00, 0x00, 0x00]);
         ElfExecutable::fixup_relative_calls(true, &mut bpf_functions, &mut prog).unwrap();
         let name = "function_5".to_string();
@@ -1068,7 +1123,7 @@ mod test {
         expected = "called `Result::unwrap()` on an `Err` value: RelativeJumpOutOfBounds(29)"
     )]
     fn test_fixup_relative_calls_out_of_bounds_forward() {
-        let mut bpf_functions: BTreeMap<u32, (usize, String)> = BTreeMap::new();
+        let mut bpf_functions = BTreeMap::new();
         // call +5
         #[rustfmt::skip]
         let mut prog = vec![
@@ -1098,7 +1153,7 @@ mod test {
         expected = "called `Result::unwrap()` on an `Err` value: RelativeJumpOutOfBounds(34)"
     )]
     fn test_fixup_relative_calls_out_of_bounds_back() {
-        let mut bpf_functions: BTreeMap<u32, (usize, String)> = BTreeMap::new();
+        let mut bpf_functions = BTreeMap::new();
         // call -7
         #[rustfmt::skip]
         let mut prog = vec![
@@ -1198,18 +1253,8 @@ mod test {
     fn test_writable_data_section() {
         let elf_bytes =
             std::fs::read("tests/elfs/writable_data_section.so").expect("failed to read elf file");
-
-        assert!(ElfExecutable::load(Config::default(), &elf_bytes, syscall_registry()).is_ok());
-
-        ElfExecutable::load(
-            Config {
-                reject_all_writable_sections: true,
-                ..Config::default()
-            },
-            &elf_bytes,
-            syscall_registry(),
-        )
-        .expect("validation failed");
+        ElfExecutable::load(Config::default(), &elf_bytes, syscall_registry())
+            .expect("validation failed");
     }
 
     #[test]
@@ -1217,14 +1262,24 @@ mod test {
     fn test_bss_section() {
         let elf_bytes =
             std::fs::read("tests/elfs/bss_section.so").expect("failed to read elf file");
-        ElfExecutable::load(
-            Config {
-                reject_all_writable_sections: true,
-                ..Config::default()
-            },
-            &elf_bytes,
-            syscall_registry(),
-        )
-        .expect("validation failed");
+        ElfExecutable::load(Config::default(), &elf_bytes, syscall_registry())
+            .expect("validation failed");
+    }
+
+    #[cfg(all(not(windows), target_arch = "x86_64"))]
+    #[test]
+    fn test_size() {
+        let mut file = File::open("tests/elfs/noop.so").expect("file open failed");
+        let mut elf_bytes = Vec::new();
+        file.read_to_end(&mut elf_bytes)
+            .expect("failed to read elf file");
+        let mut executable =
+            ElfExecutable::from_elf(&elf_bytes, None, Config::default(), syscall_registry())
+                .expect("validation failed");
+        {
+            Executable::jit_compile(&mut executable).unwrap();
+        }
+
+        assert_eq!(22792, executable.mem_size());
     }
 }
